Package: progresstest
Title: Testing Terminal Progress Bars
Version: 1.0.0
Authors@R: person("Gabor", "Csardi", , "csardi.gabort@gmail.com", role = c("aut", "cre"))
Description: Test progress bars from the progress package.
License: MIT + file LICENSE
Imports:
    progress,
    Rcpp
LinkingTo:
    progress,
    Rcpp
