Package: progress
Title: Terminal Progress Bars
Version: 1.2.3
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Rich", "FitzJohn", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Configurable Progress bars, they may include percentage,
    elapsed time, and/or the estimated completion time. They work in
    terminals, in 'Emacs' 'ESS', 'RStudio', 'Windows' 'Rgui' and the
    'macOS' 'R.app'. The package also provides a 'C++' 'API', that works
    with or without 'Rcpp'.
License: MIT + file LICENSE
URL: https://github.com/r-lib/progress#readme,
        http://r-lib.github.io/progress/
BugReports: https://github.com/r-lib/progress/issues
Depends: R (>= 3.6)
Imports: crayon, hms, prettyunits, R6
Suggests: Rcpp, testthat (>= 3.0.0), withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-12-05 09:33:10 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Rich FitzJohn [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2023-12-06 10:30:02 UTC
