# This file was generated by Rcpp::compileAttributes
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

test_progress <- function(formatSEXP = "[:bar] :percent ") {
    .Call('progresstest_test_progress', PACKAGE = 'progresstest', formatSEXP)
}

test_units <- function() {
    .Call('progresstest_test_units', PACKAGE = 'progresstest')
}

