# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

test_progress <- function(formatSEXP = "[:bar] :percent ") {
    .Call('_progresstest_test_progress', PACKAGE = 'progresstest', formatSEXP)
}

test_is_option_enabled <- function() {
    .Call('_progresstest_test_is_option_enabled', PACKAGE = 'progresstest')
}

test_units <- function() {
    .Call('_progresstest_test_units', PACKAGE = 'progresstest')
}

